94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
92 lines
2.3 KiB
Python
92 lines
2.3 KiB
Python
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
|
|
from sglang.srt.utils import is_cpu, next_power_of_2
|
|
|
|
_is_cpu = is_cpu()
|
|
|
|
if _is_cpu:
|
|
from sgl_kernel import fill_accept_out_cache_loc_cpu, fill_bonus_tokens_cpu
|
|
|
|
|
|
@triton.jit
|
|
def fill_bonus_tokens(
|
|
accept_tokens,
|
|
accept_lens,
|
|
bonus_tokens_ptr,
|
|
accept_stride: tl.constexpr,
|
|
):
|
|
# NOTE: we cannot fuse any in-place operations of `accept_lens` inside this kernel
|
|
# because this kernel reads accept_lens
|
|
pid = tl.program_id(axis=0)
|
|
# `accept_lens` includes the bonus token; the last accepted slot is at -1.
|
|
accept_len = tl.load(accept_lens + pid)
|
|
|
|
# accept_stride = per-req width of accept_tokens (= accept_index.shape[1]).
|
|
bonus_token_idx = accept_stride * pid + accept_len - 1
|
|
bonus_token = tl.load(accept_tokens + bonus_token_idx)
|
|
tl.store(bonus_tokens_ptr + pid, bonus_token)
|
|
|
|
|
|
def fill_bonus_tokens_func(
|
|
accept_tokens: torch.Tensor,
|
|
accept_lens: torch.Tensor,
|
|
bonus_tokens: torch.Tensor, # mutable
|
|
accept_stride: int,
|
|
batch_size: int,
|
|
):
|
|
if _is_cpu:
|
|
fill_bonus_tokens_cpu(
|
|
accept_tokens,
|
|
accept_lens,
|
|
bonus_tokens,
|
|
accept_stride,
|
|
)
|
|
return
|
|
fill_bonus_tokens[(batch_size,)](
|
|
accept_tokens,
|
|
accept_lens,
|
|
bonus_tokens,
|
|
accept_stride,
|
|
)
|
|
|
|
|
|
@triton.jit
|
|
def fill_accept_out_cache_loc(
|
|
accept_index,
|
|
out_cache_loc,
|
|
accept_out_cache_loc,
|
|
size_upper: tl.constexpr,
|
|
):
|
|
pid = tl.program_id(axis=0)
|
|
offset = tl.arange(0, size_upper)
|
|
|
|
masks = (tl.load(accept_index + offset, offset < pid, other=-1) != -1).to(tl.int64)
|
|
dst = tl.sum(masks)
|
|
src = tl.load(accept_index + pid)
|
|
if src > -1:
|
|
value = tl.load(out_cache_loc + src)
|
|
tl.store(accept_out_cache_loc + dst, value)
|
|
|
|
|
|
def fill_accept_out_cache_loc_func(
|
|
accept_index: torch.Tensor,
|
|
out_cache_loc: torch.Tensor,
|
|
accept_out_cache_loc: torch.Tensor, # mutable
|
|
size: int,
|
|
):
|
|
if _is_cpu:
|
|
fill_accept_out_cache_loc_cpu(
|
|
accept_index,
|
|
out_cache_loc,
|
|
accept_out_cache_loc,
|
|
)
|
|
return
|
|
fill_accept_out_cache_loc[(size,)](
|
|
accept_index,
|
|
out_cache_loc,
|
|
accept_out_cache_loc,
|
|
next_power_of_2(size),
|
|
)
|